﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;

namespace VectorGrafik
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
   
   
    public partial class MainWindow : Window
    {

        private List<Point> p = new List<Point>();
        private ObjectTyp newObjectTyp;
        private int drawStep;
        internal Graphic2D Grafic;
        private Style style;
        private string fileName = "";
        private Point? dragStartPoint = null;
        
        public MainWindow()
        {
            InitializeComponent();
            Grafic = new Graphic2D(canvas);
            style = (Style)this.FindResource("DesignerItemStyle");
            canvas.StyleForObjects = style;
            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.Open, Open_Executed));
            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.SaveAs, SaveAS_Executed));
            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.Save, Save_Executed));
            this.CommandBindings.Add(new CommandBinding(ApplicationCommands.New, New_Executed));
        }

        private void canvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            switch (Grafic.State)
            {
                case GraficState.grfNormal:
                    canvas.DeselectAll();
                    this.dragStartPoint = new Point?(e.GetPosition(this.canvas));
                    break;
                case GraficState.grfDraw:
                    DesignerItem designerItem = (DesignerItem)canvas.Children[canvas.Children.Count - 1];
                    for (int i = drawStep; i < designerItem.NumberOfPoints; i++)
                    {
                        p.Add(e.GetPosition(canvas));
                    }
                    Grafic.DeleteLastElement();
                    switch (newObjectTyp)
                    {
                        case ObjectTyp.MyLine:
                            Grafic.CreateNewLine(p[0], p[1]);
                            break;
                        case ObjectTyp.MyRectangle:
                            Grafic.CreateNewRectangle(p[0], p[1]);
                            break;
                        case ObjectTyp.MyEllipse:
                            Grafic.CreateNewEllipse(p[0], p[1]);
                            break;
                        case ObjectTyp.MyTriangle:
                            Grafic.CreateNewTriangle(p[0], p[1], p[2]);
                            break;
                    }
                    drawStep++;
                    break;
            }
        }
        private void canvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            switch (Grafic.State)
            {
                case GraficState.grfNormal:
                    canvas.ReleaseMouseCapture();
                    break;
                case GraficState.grfDraw:
                    DesignerItem designerItem = (DesignerItem)canvas.Children[canvas.Children.Count - 1];
                    if (drawStep == designerItem.NumberOfPoints)
                    {
                        Grafic.State = GraficState.grfNormal;
                        drawStep = 0;
                        p.Clear();
                        canvas.ReleaseMouseCapture();
                        DesignerItem cc = (DesignerItem)canvas.Children[canvas.Children.Count - 1];
                        cc.Style = style;
                    }
                    break;
                default:
                    break;
            }
        }
        private void canvas_MouseMove(object sender, MouseEventArgs e)
        {
            switch (Grafic.State)
            {
                case GraficState.grfNormal:

                        if (e.LeftButton != MouseButtonState.Pressed)
                        {
                            this.dragStartPoint = null;
                        }

                        if (this.dragStartPoint.HasValue)
                        {   
                            AdornerLayer adornerLayer = AdornerLayer.GetAdornerLayer(this.canvas);
                            if (adornerLayer != null)
                            {
                                RubberbandAdorner adorner = new RubberbandAdorner(this.canvas, dragStartPoint);
                                if (adorner != null)
                                {
                                    adornerLayer.Add(adorner);
                                }
                            }
                        }
                    break;
                case GraficState.grfDraw:
                    canvas.CaptureMouse();
                    
                    if (drawStep >= 1)
                    {
                        Grafic.DeleteLastElement();
                        p[drawStep] = e.GetPosition(canvas);
                        switch (newObjectTyp)
                        {
                            case ObjectTyp.MyLine:
                                Grafic.CreateNewLine(p[0], p[1]);
                                break;
                            case ObjectTyp.MyRectangle:
                                Grafic.CreateNewRectangle(p[0], p[1]);
                                break;
                            case ObjectTyp.MyEllipse:
                                Grafic.CreateNewEllipse(p[0], p[1]);
                                break;
                            case ObjectTyp.MyTriangle:
                                Grafic.CreateNewTriangle(p[0], p[1], p[2]);
                                break;
                        }
                    }
                    break;
                default:
                    break;
            }
        }

        private void MenuItem_Click_Line(object sender, RoutedEventArgs e)
        {
            Grafic.State = GraficState.grfDraw;
            drawStep = 0;
            newObjectTyp = ObjectTyp.MyLine;
            Grafic.CreateNewLine(new Point(), new Point());
        }
        private void MenuItem_Click_Rectangle(object sender, RoutedEventArgs e)
        {
            Grafic.State = GraficState.grfDraw;
            drawStep = 0;
            newObjectTyp = ObjectTyp.MyRectangle;
            Grafic.CreateNewRectangle(new Point(), new Point());
        }

        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
            AboutWindow formAbout = new AboutWindow();
            formAbout.ShowDialog();
        }


       

        private void MenuItem_Click_2(object sender, RoutedEventArgs e)
        {
            foreach (DesignerItem child in canvas.Children)
            {
                child.IsSelected = true;
            }
        }

        private void MenuItem_Click_1(object sender, RoutedEventArgs e)
        {
            foreach (DesignerItem child in canvas.Children)
            {
                child.IsSelected = false;
            }
        }

        private void MenuItem_Click_Ellipse(object sender, RoutedEventArgs e)
        {
            Grafic.State = GraficState.grfDraw;
            drawStep = 0;
            newObjectTyp = ObjectTyp.MyEllipse;
            Grafic.CreateNewEllipse(new Point(), new Point());
        }

        private void MenuItem_Click_4(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void MenuItem_Click_Triangle(object sender, RoutedEventArgs e)
        {
            Grafic.State = GraficState.grfDraw;
            drawStep = 0;
            newObjectTyp = ObjectTyp.MyTriangle;
            Grafic.CreateNewTriangle(new Point(), new Point(), new Point());
        }


        private void Save_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if (fileName == "") SaveAS_Executed(sender, e);
            else Grafic.Save(fileName);
        }

        private void SaveAS_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            fileName = Grafic.SaveAs();
            if (fileName != "")
            {
                this.Title = "VectorGraficWPF " + fileName;                
            }
        }

        private void Open_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            fileName = Grafic.Open();
            if (fileName != "")
            {
                this.Title = "VectorGraficWPF " + fileName;
            }
        }
        private void New_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            MessageBoxResult result = MessageBox.Show("Wollen Sie Änderungen speichern?", "Neues Dokument", MessageBoxButton.YesNoCancel);
            switch (result)
            {
                case MessageBoxResult.Cancel:
                    break;
                case MessageBoxResult.No:
                    // keine weiteren Aktionen       
                    break;
                case MessageBoxResult.Yes:
                    Save_Executed(sender, e);
                    break;
                default:
                    break;
            }
            if (result != MessageBoxResult.Cancel)
            {
                fileName = "";
                this.Title = "VectorGraficWPF " + "Unbenannt.grf";
                Grafic.Clear();
            }
        }
    }
}
